@@ -21,44 +21,53 @@ const GLib = imports.gi.GLib;
 const Gnome = imports.gi.GnomeDesktop;
 const GObject = imports.gi.GObject;
 const Gtk = imports.gi.Gtk;
+const GWeather = imports.gi.GWeather;
 
-const Forecast = imports.app.forecast;
-const WForecast = imports.app.weeklyForecast;
+const WorldView = imports.app.world;
+const HourlyForecast = imports.app.hourlyForecast;
+const DailyForecast = imports.app.dailyForecast;
 const Util = imports.misc.util;
 
 const SPINNER_SIZE = 128;
 
 const SCROLLING_ANIMATION_TIME = 400000; //us
 
+const UPDATED_TIME_TIMEOUT = 60; //s
+
 var WeatherWidget = GObject.registerClass({
     Template: 'resource:///org/gnome/Weather/weather-widget.ui',
-    InternalChildren: ['contentFrame', 'outerGrid', 'conditionsImage',
-                       'temperatureLabel', 'conditionsLabel',
-                       'timeLabel', 'timeGrid', 'forecastStack',
-                       'leftButton', 'rightButton',
-                       'forecast-today-grid', 'forecast-tomorrow-grid',
-                       'forecast-today', 'forecast-tomorrow'],
+    InternalChildren: ['contentFrame', 'outerBox',
+                       'conditionsImage', 'placesButton', 'placesLabel',
+                       'temperatureLabel', 'apparentLabel',
+                       'forecastStack', 'leftBox', 'leftButton', 'rightBox', 'rightButton',
+                       'forecast-hourly', 'forecast-hourly-viewport',
+                       'forecast-daily', 'forecast-daily-viewport',
+                       'updatedTimeLabel', 'attributionLabel'],
 }, class WeatherWidget extends Gtk.Frame {
 
-    _init(params) {
+    _init(application, window, params) {
         super._init(Object.assign({
             shadow_type: Gtk.ShadowType.NONE,
             name: 'weather-page'
         }, params));
 
-        this._currentStyle = null;
         this._info = null;
 
-        this._weeklyForecasts = new WForecast.WeeklyForecastFrame();
-        this._outerGrid.attach(this._weeklyForecasts, 1, 0, 1, 2);
+        this._worldView = new WorldView.WorldContentView(application, window);
+        this._placesButton.set_popover(this._worldView);
 
         this._forecasts = { };
 
-        for (let t of ['today', 'tomorrow']) {
-            let box = new Forecast.ForecastBox({ hexpand: false });
+        for (let t of ['hourly', 'daily']) {
+            let box;
+            if (t == 'hourly') {
+                box = new HourlyForecast.HourlyForecastFrame();
+            } else {
+                box = new DailyForecast.DailyForecastFrame();
+            }
 
             this._forecasts[t] = box;
-            this['_forecast_' + t + '_grid'].add(box);
+            this['_forecast_' + t + '_viewport'].add(box);
 
             let fsw = this['_forecast_' + t];
             let hscrollbar = fsw.get_hscrollbar();
@@ -99,22 +108,34 @@ var WeatherWidget = GObject.registerClass({
 
             this._beginScrollAnimation(target);
         });
+
+        this._updatedTime = null;
+        this._updatedTimeTimeoutId = 0;
+
+        this.connect('destroy', () => this._onDestroy());
+    }
+
+    _onDestroy() {
+        if (this._updatedTimeTimeoutId) {
+            GLib.Source.remove(this._updatedTimeTimeoutId);
+            this._updatedTimeTimeoutId = 0;
+        }
     }
 
     _syncLeftRightButtons() {
         let hadjustment = this._forecastStack.visible_child.get_hadjustment();
         if ((hadjustment.get_upper() - hadjustment.get_lower()) == hadjustment.page_size) {
-            this._leftButton.set_sensitive(false);
-            this._rightButton.set_sensitive(false);
+            this._leftBox.hide();
+            this._rightBox.hide();
         } else if (hadjustment.value == hadjustment.get_lower()){
-            this._leftButton.set_sensitive(false);
-            this._rightButton.set_sensitive(true);
+            this._leftBox.hide();
+            this._rightBox.show();
         } else if (hadjustment.value >= (hadjustment.get_upper() - hadjustment.page_size)){
-            this._leftButton.set_sensitive(true);
-            this._rightButton.set_sensitive(false);
+            this._leftBox.show();
+            this._rightBox.hide();
         } else {
-            this._leftButton.set_sensitive(true);
-            this._rightButton.set_sensitive(true);
+            this._leftBox.show();
+            this._rightBox.show();
         }
     }
 
@@ -147,7 +168,7 @@ var WeatherWidget = GObject.registerClass({
     }
 
     clear() {
-        for (let t of ['today', 'tomorrow'])
+        for (let t of ['hourly', 'daily'])
             this._forecasts[t].clear();
 
         if (this._tickId) {
@@ -156,48 +177,97 @@ var WeatherWidget = GObject.registerClass({
         }
     }
 
-    _getStyleClass(info) {
-        let icon = info.get_icon_name();
-        let name = icon.replace(/(-\d{3})/, "");
-        return name;
-    }
-
-    setTimeVisible(visible) {
-        this._timeGrid.visible = visible;
-    }
-
-    setTime(time) {
-        this._timeLabel.label = time;
+    getForecastStack() {
+        return this._forecastStack;
     }
 
     update(info) {
         this._info = info;
 
-        this._conditionsLabel.label = Util.getWeatherConditions(info);
-        this._temperatureLabel.label = info.get_temp_summary();
+        let location = info.location;
+        let city = location;
+        if (location.get_level() == GWeather.LocationLevel.WEATHER_STATION)
+            city = location.get_parent();
 
-        this._conditionsImage.icon_name = info.get_symbolic_icon_name();
-        let context = this._contentFrame.get_style_context();
-        if (this._currentStyle)
-            context.remove_class(this._currentStyle);
-        this._currentStyle = this._getStyleClass(info);
-        context.add_class(this._currentStyle);
+        let country = city.get_parent();
+        while (country && country.get_level() > GWeather.LocationLevel.COUNTRY)
+            country = country.get_parent();
+
+        if (country)
+            this._placesLabel.set_text(city.get_name() + ', ' + country.get_name());
+        else
+            this._placesLabel.set_text(city.get_name());
+
+        this._worldView.refilter();
+
+        this._conditionsImage.iconName = info.get_icon_name() + '-large';
+
+        const [, tempValue] = info.get_value_temp(GWeather.TemperatureUnit.DEFAULT);
+        this._temperatureLabel.label = '%d°'.format(Math.round(tempValue));
+
+        const [, apparentValue] = info.get_value_apparent(GWeather.TemperatureUnit.DEFAULT);
+        this._apparentLabel.label = _('Feels like %.0f°').format(apparentValue);
 
         let forecasts = info.get_forecast_list();
         let tz = GLib.TimeZone.new(info.location.get_timezone().get_tzid());
-        for (let t of ['today', 'tomorrow'])
-            this._forecasts[t].update(forecasts, tz, t);
+        for (let t of ['hourly', 'daily'])
+            this._forecasts[t].update(forecasts, tz);
 
-        if (!this._forecasts['today'].hasForecastInfo() && this._forecasts['tomorrow'].hasForecastInfo())
-            this._forecastStack.set_visible_child_name('tomorrow');
+        if (this._updatedTimeTimeoutId)
+            GLib.Source.remove(this._updatedTimeTimeoutId);
 
-        // FIXME: This doesn't make sense, since the above code assumes forecasts.length != 0.
-        if (forecasts.length == 0) {
-            this._weeklyForecasts.hide();
-        } else {
-            this._weeklyForecasts.show();
-            this._weeklyForecasts.update(forecasts);
-        }
+        this._updatedTime = Date.now();
+        this._updatedTimeLabel.label = this._formatUpdatedTime();
+
+        this._updatedTimeTimeoutId = GLib.timeout_add_seconds(
+            GLib.PRIORITY_DEFAULT,
+            UPDATED_TIME_TIMEOUT, () => {
+                this._updatedTimeLabel.label = this._formatUpdatedTime();
+                return GLib.SOURCE_CONTINUE;
+            }
+        );
+
+        this._attributionLabel.label = info.get_attribution();
+    }
+
+    _formatUpdatedTime() {
+        if (this._updatedTime == null)
+            return '';
+
+        const milliseconds = Date.now() - this._updatedTime;
+
+        const seconds = milliseconds / 1000;
+        if (seconds < 60)
+            return _('Updated just now.');
+
+        const minutes = seconds / 60;
+        if (minutes < 60)
+            return ngettext(
+                'Updated %d minute ago.',
+                'Updated %d minutes ago.', minutes).format(minutes);
+
+        const hours = minutes / 60;
+        if (hours < 24)
+            return ngettext(
+                'Updated %d hour ago.',
+                'Updated %d hours ago.', hours).format(hours);
+
+        const days = hours / 24;
+        if (days < 7)
+            return ngettext(
+                'Updated %d day ago.',
+                'Updated %d days ago.', days).format(days);
+
+        const weeks = days / 7;
+        if (days < 30)
+            return ngettext(
+                'Updated %d week ago.',
+                'Updated %d weeks ago.', weeks).format(weeks);
+
+        const months = days / 30;
+        return ngettext(
+            'Updated %d month ago.',
+            'Updated %d months ago.', months).format(months);
     }
 });
 
@@ -206,10 +276,10 @@ var WeatherView = GObject.registerClass({
     InternalChildren: ['spinner']
 }, class WeatherView extends Gtk.Stack {
 
-    _init(params) {
+    _init(application, window, params) {
         super._init(params);
 
-        this._infoPage = new WeatherWidget();
+        this._infoPage = new WeatherWidget(application, window);
         this.add_named(this._infoPage, 'info');
 
         this._info = null;
@@ -264,40 +334,11 @@ var WeatherView = GObject.registerClass({
     _onUpdate(info) {
         this._infoPage.clear();
         this._infoPage.update(info);
-        this._updateTime();
         this._spinner.stop();
         this.visible_child_name = 'info';
     }
 
-    setTimeVisible(visible) {
-        if (this._clockHandlerId && !visible) {
-            this._wallClock.disconnect(this._clockHandlerId);
-            this._clockHandlerId = 0;
-        }
-
-        if (!this._clockHandlerId && visible) {
-            this._clockHandlerId = this._wallClock.connect('notify::clock',  () => {
-                this._updateTime();
-            });
-        }
-
-        this._infoPage.setTimeVisible(visible);
-    }
-
-    _updateTime() {
-        this._infoPage.setTime(this._getTime());
-    }
-
-    _getTime() {
-        if (this._info != null) {
-            let location = this._info.location;
-            let tz = GLib.TimeZone.new(location.get_timezone().get_tzid());
-            let dt = GLib.DateTime.new_now(tz);
-
-            return this._wallClock.string_for_datetime (dt,
-                                                        this._desktopSettings.get_enum('clock-format'),
-                                                        false, false, false);
-        }
-        return null;
+    getInfoPage() {
+        return this._infoPage;
     }
 });
